==> iversioning-0.0.11.tar.gz: lib/iversioning.dart <==
library iversioning;

import 'package:flutter/cupertino.dart';
import 'package:iversioning/data/web/iversioning_api.dart';
import 'package:iversioning/ui/iversion_prompt_dialog.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/code/nui_device_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/model/interfaces.dart';
import 'package:nui_web/nui_web.dart';
import 'data/web/iversioning_json.dart';
import 'dart:math' as math;

abstract class IVersioningInterface{
  Future<DataResult<NUIAppVersionResponse>> getSupportedAppVersions(); //Get supported app versions
  Future<DataResult<NUIAppVersionActionResponse>> checkAppVersion(BuildContext context, {bool takeAction: true}); //Get supported app versions
  Future<bool> isLatestVersion();
  Future<NUIAppVersionItem> getLatestAppVersion();
}

class IVersioning extends IVersioningInterface{
  static const String _MODULE = "IVersioning";

  String _baseUrl;
  IVersioningApi _api;
  IDialogTheme _dialogTheme;
  static IVersioning _instance;
  String _appVersion;
  String _apiKey;

  IVersioning._builder(IVersioningBuilder builder){
    _baseUrl = builder._baseUrl;
    _api = IVersioningApi();
    _appVersion = builder._appVersion;
    _dialogTheme = builder._dialogTheme;
    _apiKey = builder._apiKey;
    _instance = this;

    _buildNUIWeb();
  }

  void _buildNUIWeb() async{
    final deviceInfo = await NUIDeviceUtil.getDeviceInfo();
    NUIWebBuilder()
        .module(IVersioningApi.MODULE)
        .baseURL(_baseUrl)
        .headers((){

      return {
        "DeviceId": deviceInfo.uuid,
        "Content-Type": "application/json",
        "X-DeviceOs": deviceInfo.platform,
        "X-DeviceOs-Version": deviceInfo.version,
        "Device": deviceInfo.model,
        "Version": _appVersion,
        "Api-Key": _apiKey
      };
    })
    .build();
  }

  static IVersioning get(){
    if(_instance == null) throwNUI(_MODULE, "Instance has not been created with builder");
    return _instance;
  }

  Future<DataResult<NUIAppVersionResponse>> getSupportedAppVersions() async{
    final deviceInfo = await NUIDeviceUtil.getDeviceInfo();
    final result = await _api.getSupportedAppVersions(_appVersion, deviceInfo.platform);
    return result;
  }

  Future<DataResult<NUIAppVersionActionResponse>> checkAppVersion(BuildContext context, {bool takeAction: true}) async{
    final deviceInfo = await NUIDeviceUtil.getDeviceInfo();
    final result = await _api.checkAppVersion(_appVersion, deviceInfo.platform);

    if(result.success && result.data != null && takeAction == true) {
      await _processAppVersionAction(context, result.data);
    }

    return result;
  }

  Future<bool> _processAppVersionAction(BuildContext context, NUIAppVersionActionResponse response) async{
    response.colorTheme = response.colorTheme ?? _dialogTheme;
    response.colorTheme.copyFrom(_dialogTheme);

    switch(response.action){
      case NUIAppVersionItem.LEVEL_NO_PROMPT : {
        return false;
      }
      case NUIAppVersionItem.LEVEL_PROMPT : {
        showIVersionPromptDialog(context, response);
        return true;
      }
      case NUIAppVersionItem.LEVEL_FORCE_UPDATE : {
        showIVersionPromptDialog(context, response);
        return true;
      }
      case NUIAppVersionItem.LEVEL_HIDDEN : {
        return false;
      }
      case NUIAppVersionItem.LEVEL_NONE : {
        //No action required
        return false;
      }
    }
    return false;
  }

  int _compareNewAndOldVersion(String newVersion, String oldVersion){
    final newSplitted = newVersion.split(".");
    final oldSplitted = oldVersion.split(".");

    int difference;
    for(int i = 0; i < math.max(newSplitted.length, oldSplitted.length); i ++){
      final newInt = int.parse(newSplitted.getOrNull(i) ?? "0");
      final oldInt = int.parse(oldSplitted.getOrNull(i) ?? "0");

      if(newInt != oldInt){
        //A difference found
        difference = newInt - oldInt;
        break;
      }
    }

    //Positive indicates that a newer version is available, negative indicates an older version is used, zero indicates no version change
    return difference ?? 0;
  }

  @override
  Future<NUIAppVersionItem> getLatestAppVersion() async{
    final newVersions = await getSupportedAppVersions();
    if(newVersions.success && newVersions.data != null){
      NUIAppVersionItem latestVersion;
      for(var v in newVersions.data.versions){
        if(latestVersion == null || _compareNewAndOldVersion(v.version, latestVersion.version) > 0){
          latestVersion = v;
        }
      }
      return latestVersion;
    }
    return null;
  }

  @override
  Future<bool> isLatestVersion() async{
    final latestVersion = await getLatestAppVersion();
    if(latestVersion == null) return true;

    if(_compareNewAndOldVersion(latestVersion.version, _appVersion) == 0){
      return true;
    }
    return false;
  }
}

class IVersioningBuilder{
  String _baseUrl;
  String _appVersion;
  IDialogTheme _dialogTheme;
  String _apiKey;

  IVersioningBuilder baseUrl(String url){
    _baseUrl = url;
    return this;
  }

  IVersioningBuilder apiKey(String key){
    _apiKey = key;
    return this;
  }

  IVersioningBuilder appVersion(String version){
    _appVersion = version;
    return this;
  }

  IVersioningBuilder dialogTheme(IDialogTheme theme){
    _dialogTheme = theme;
    return this;
  }

  IVersioning build(){
    return IVersioning._builder(this);
  }
}

==> iversioning-0.0.14.tar.gz: lib/iversioning.dart <==
library iversioning;

import 'package:flutter/cupertino.dart';
import 'package:iversioning/data/web/iversioning_api.dart';
import 'package:iversioning/ui/iversion_prompt_dialog.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/code/nui_device_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/model/interfaces.dart';
import 'package:nui_core/ui/component/nui_dialog_widget.dart';
import 'package:nui_core/ui/nui_navigator.dart';
import 'package:nui_web/nui_web.dart';
import 'package:url_launcher/url_launcher.dart';
import 'data/web/iversioning_json.dart';
import 'dart:math' as math;

typedef IVersioningWidgetBuilder = Widget Function(BuildContext context, NUIAppVersionActionResponse action);

abstract class IVersioningInterface{
  Future<DataResult<NUIAppVersionResponse>> getSupportedAppVersions(); //Get supported app versions
  Future<DataResult<NUIAppVersionActionResponse>> checkAppVersion(BuildContext context, {bool takeAction: true}); //Get supported app versions
  Future<bool> isLatestVersion();
  Future<NUIAppVersionItem> getLatestAppVersion();
}

class IVersioning extends IVersioningInterface{
  static const String _MODULE = "IVersioning";

  String _baseUrl;
  IVersioningApi _api;
  IDialogTheme _dialogTheme;
  static IVersioning _instance;
  String _appVersion;
  String _apiKey;
  IVersioningWidgetBuilder _dialogBuilder;

  IVersioning._builder(IVersioningBuilder builder){
    _baseUrl = builder._baseUrl;
    _api = IVersioningApi();
    _appVersion = builder._appVersion;
    _dialogTheme = builder._dialogTheme;
    _apiKey = builder._apiKey;
    _dialogBuilder = builder._dialogBuilder;
    _instance = this;

    _buildNUIWeb();
  }

  void _buildNUIWeb() async{
    final deviceInfo = await NUIDeviceUtil.getDeviceInfo();
    NUIWebBuilder()
        .module(IVersioningApi.MODULE)
        .baseURL(_baseUrl)
        .headers((){

      return {
        "DeviceId": deviceInfo.uuid,
        "Content-Type": "application/json",
        "X-DeviceOs": deviceInfo.platform,
        "X-DeviceOs-Version": deviceInfo.version,
        "Device": deviceInfo.model,
        "Version": _appVersion,
        "Api-Key": _apiKey
      };
    })
    .build();
  }

  static IVersioning get instance{
    if(_instance == null) throwNUI(_MODULE, "Instance has not been created with builder");
    return _instance;
  }

  Future<DataResult<NUIAppVersionResponse>> getSupportedAppVersions() async{
    final deviceInfo = await NUIDeviceUtil.getDeviceInfo();
    final result = await _api.getSupportedAppVersions(_appVersion, deviceInfo.platform);
    return result;
  }

  Future<DataResult<NUIAppVersionActionResponse>> checkAppVersion(BuildContext context, {bool takeAction: true}) async{
    final deviceInfo = await NUIDeviceUtil.getDeviceInfo();
    final result = await _api.checkAppVersion(_appVersion, deviceInfo.platform);

    if(result.success && result.data != null && takeAction == true) {
      await _processAppVersionAction(context, result.data);
    }

    return result;
  }

  void _showDialog(BuildContext context, NUIAppVersionActionResponse response){
    if(_dialogBuilder == null) {
      showIVersionPromptDialog(context, response);
    }
    else{
      final dialog = _dialogBuilder(context, response);
      showNUIDialog(
          transition: PageTransition.BLUR_BACKGROUND,
          transitionDuration: Duration(milliseconds: 250),
          context: context,
          dismissible: false,
          content: (context) => dialog
      );
    }
  }

  Future<bool> _processAppVersionAction(BuildContext context, NUIAppVersionActionResponse response) async{
    response.colorTheme = response.colorTheme ?? _dialogTheme;
    response.colorTheme.copyFrom(_dialogTheme);

    switch(response.action){
      case NUIAppVersionItem.LEVEL_NO_PROMPT : {
        return false;
      }
      case NUIAppVersionItem.LEVEL_PROMPT : {
        _showDialog(context, response);
        return true;
      }
      case NUIAppVersionItem.LEVEL_FORCE_UPDATE : {
        _showDialog(context, response);
        return true;
      }
      case NUIAppVersionItem.LEVEL_HIDDEN : {
        return false;
      }
      case NUIAppVersionItem.LEVEL_NONE : {
        //No action required
        return false;
      }
    }
    return false;
  }

  void launchForUpdate(NUIAppVersionActionResponse response) async {
    if (!isNullOrEmpty(response.downloadLink)) {
      logNUI(_MODULE, "Download link: ${response.downloadLink}");
      if (await canLaunch(response.downloadLink)) {
        await launch(response.downloadLink);
      } else {
        logNUI(_MODULE, "Failed to open download link");
      }
    }
  }

  int _compareNewAndOldVersion(String newVersion, String oldVersion){
    final newSplitted = newVersion.split(".");
    final oldSplitted = oldVersion.split(".");

    int difference;
    for(int i = 0; i < math.max(newSplitted.length, oldSplitted.length); i ++){
      final newInt = int.parse(newSplitted.getOrNull(i) ?? "0");
      final oldInt = int.parse(oldSplitted.getOrNull(i) ?? "0");

      if(newInt != oldInt){
        //A difference found
        difference = newInt - oldInt;
        break;
      }
    }

    //Positive indicates that a newer version is available, negative indicates an older version is used, zero indicates no version change
    return difference ?? 0;
  }

  @override
  Future<NUIAppVersionItem> getLatestAppVersion() async{
    final newVersions = await getSupportedAppVersions();
    if(newVersions.success && newVersions.data != null){
      NUIAppVersionItem latestVersion;
      for(var v in newVersions.data.versions){
        if(latestVersion == null || _compareNewAndOldVersion(v.version, latestVersion.version) > 0){
          latestVersion = v;
        }
      }
      return latestVersion;
    }
    return null;
  }

  @override
  Future<bool> isLatestVersion() async{
    final latestVersion = await getLatestAppVersion();
    if(latestVersion == null) return true;

    if(_compareNewAndOldVersion(latestVersion.version, _appVersion) == 0){
      return true;
    }
    return false;
  }
}

class IVersioningBuilder{
  String _baseUrl;
  String _appVersion;
  IDialogTheme _dialogTheme;
  String _apiKey;
  IVersioningWidgetBuilder _dialogBuilder;

  IVersioningBuilder baseUrl(String url){
    _baseUrl = url;
    return this;
  }

  IVersioningBuilder dialogBuilder(IVersioningWidgetBuilder builder){
    _dialogBuilder = builder;
    return this;
  }

  IVersioningBuilder apiKey(String key){
    _apiKey = key;
    return this;
  }

  IVersioningBuilder appVersion(String version){
    _appVersion = version;
    return this;
  }

  IVersioningBuilder dialogTheme(IDialogTheme theme){
    _dialogTheme = theme;
    return this;
  }

  IVersioning build(){
    return IVersioning._builder(this);
  }
}
